#Copyright (c) 2015 3D Control Systems LTD

#3DPrinterOS client is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#3DPrinterOS client is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Vladimir Avdeev <another.vic@yandex.ru>

import base64
import logging
import collections

import log
import config
import downloader


class BaseSender:

    def __init__(self, profile, usb_info):
        self.logger = logging.getLogger(__name__)
        self.stop_flag = False
        self.profile = profile
        self.usb_info = usb_info
        self.position = [0, 0, 0, 0]  # X, Y, Z, E
        self.temps = [0,0]
        self.target_temps = [0,0]
        self.total_gcodes = None
        self.buffer = collections.deque()
        self.downloader = None
        self.current_line_number = 0
        self.loading_gcodes_flag = False
        self.cancel_after_loading_flag = False
        self.pause_flag = False
        self.is_running = True

    def set_total_gcodes(self, length):
        raise NotImplementedError

    def load_gcodes(self, gcodes):
        raise NotImplementedError

    def unbuffered_gcodes(self, gcodes):
        raise NotImplementedError

    def preprocess_gcodes(self, gcodes):
        gcodes = gcodes.replace("\r", "")
        gcodes = gcodes.split("\n")
        gcodes = filter(lambda item: item, gcodes)
        if gcodes:
            while gcodes[-1] in ("\n", "\n", "\t", " ", "", None):
                line = gcodes.pop()
                self.logger.info("Removing corrupted line '%s' from gcodes tail" % line)
        self.logger.info('Got %d gcodes to print.' % len(gcodes))
        return gcodes

    def gcodes(self, gcodes_or_link, is_link = False):
        if is_link:
            if self.is_downloading():
                self.logger.error('Download command received while downloading processing. Aborting...')
                return False
            else:
                self.downloader = downloader.Downloader(self, gcodes_or_link)
                self.downloader.start()
        else:
            gcodes = base64.b64decode(gcodes_or_link)
            self.unbuffered_gcodes(gcodes)

    def get_position(self):
        return self.position

    def get_temps(self):
        return self.temps

    def get_target_temps(self):
        return self.target_temps

    def cancel_download(self):
        if self.is_downloading():
            self.logger.info("Canceling downloading")
            self.downloader.cancel()
            return True

    def pause(self):
        self.pause_flag = True

    def unpause(self):
        self.pause_flag = False

    def close(self):
        self.stop_flag = True
        self.is_running = False

    def is_paused(self):
        return self.pause_flag

    def is_operational(self):
        return False

    def is_downloading(self):
        return self.downloader and self.downloader.is_alive()

    def upload_logs(self):
        self.logger.info("Sending logs")
        if log.send_logs():
            return False

    def switch_camera(self, module):
        self.logger.info('Changing camera module to %s due to server request' % module)
        printer_interface = self.find_parent_printer_interface()
        if printer_interface:
            config.get_app().camera_controller.switch_camera(module, printer_interface.printer_token)
            return True

    def restart_camera(self):
        self.logger.info('Executing camera restart command from server')
        config.get_app().camera_controller.restart_camera()

    def update_software(self):
        self.logger.info('Executing update command from server')
        config.get_app().updater.update()

    def quit_application(self):
        self.logger.info('Received quit command from server!')
        config.get_app().stop_flag = True

    def set_groups(self, groups): # a bit hacky but will work fine for now. TODO refactor me
        self.logger.info("Setting groups: " + str(groups))
        printer_interface = self.find_parent_printer_interface()
        if printer_interface:
            printer_interface.set_groups(groups)
            return True

    def set_name(self, name): # TODO FIX COPY-PASTE WITH ABOVE
        self.logger.info("Setting printer name: " + str(name))
        printer_interface = self.find_parent_printer_interface()
        if printer_interface:
            printer_interface.set_printer_name(name)
            return True

    def find_parent_printer_interface(self):
        for printer_interface in config.get_app().printer_interfaces:
            if printer_interface.usb_info == self.usb_info:
                return printer_interface

    def report_problem(self, problem_description):
        log.report_problem(problem_description)


